@@ -3,6 +3,7 @@ Python types for constant values in Procmon
 """
 
 import enum
+from collections import OrderedDict
 
 
 class RuleAction(enum.IntEnum):
@@ -196,6 +197,15 @@ class FilesystemOperation(enum.IntEnum):
     PlugAndPlay = 47  # IRP_MJ_PNP
 
 
+EventClassOperation = {
+    EventClass.Process: ProcessOperation,
+    EventClass.Registry: RegistryOperation,
+    EventClass.File_System: FilesystemOperation,
+    EventClass.Profiling: ProfilingOperation,
+    EventClass.Network: NetworkOperation
+}
+
+
 class FilesystemQueryVolumeInformationOperation(enum.IntEnum):
     QueryInformationVolume = 0x1
     QueryLabelInformationVolume = 0x2
@@ -311,6 +321,16 @@ class FilesystemLockUnlockOperation(enum.IntEnum):
     UnlockFileByKey = 0x4  # IRP_MJ_LOCK_CONTROL, FASTIO_UNLOCK_ALL_BY_KEY
 
 
+FilesystemSubOperations = {
+    FilesystemOperation.QueryVolumeInformation: FilesystemQueryVolumeInformationOperation,
+    FilesystemOperation.SetVolumeInformation: FilesystemSetVolumeInformationOperation,
+    FilesystemOperation.QueryInformationFile: FilesystemQueryInformationOperation,
+    FilesystemOperation.SetInformationFile: FilesystemSetInformationOperation,
+    FilesystemOperation.DirectoryControl: FilesysemDirectoryControlOperation,
+    FilesystemOperation.PlugAndPlay: FilesystemPnpOperation,
+    FilesystemOperation.LockUnlockFile: FilesystemLockUnlockOperation,
+}
+
 _ErrorCodeMessages = {
     0: 'SUCCESS',
     0x103: '',
@@ -463,3 +483,259 @@ _ErrorCodeMessages = {
 
 def get_error_message(error_value):
     return _ErrorCodeMessages.get(error_value, "0x{:X}".format(error_value))
+
+
+def _get_mask_string(mask, mask_strings, seperator):
+    strings = []
+    for value, string in mask_strings.items():
+        if mask & value == value:
+            strings.append(string)
+            mask &= ~value
+
+    if len(strings) == 0:
+        return ''
+    return seperator.join(strings)
+
+
+COMMON_ACCESS_MASK_STRINGS = OrderedDict([
+    (0x10000, "Delete"),
+    (0x20000, "Read Control"),
+    (0x40000, "Write DAC"),
+    (0x80000, "Write Owner"),
+    (0x100000, "Synchronize"),
+    (0x1000000, "Access System Security"),
+    (0x2000000, "Maximum Allowed"),
+])
+
+
+REGISTRY_ACCESS_MASK_MAPPING = [0x20019, 0x20006, 0x20019, 0xf003f]  # used in MapGenericMask
+REGISTRY_ACCESS_MASK_STRINGS = OrderedDict([
+    (0xf003f, "All Access"),
+    (0x2001f, "Read/Write"),
+    (0x20019, "Read"),
+    (0x20006, "Write"),
+    (0x1, "Query Value"),
+    (0x2, "Set Value"),
+    (0x4, "Create Sub Key"),
+    (0x8, "Enumerate Sub Keys"),
+    (0x10, "Notify"),
+    (0x20, "Create Link"),
+    (0x300, "WOW64_Res"),
+    (0x200, "WOW64_32Key"),
+    (0x100, "WOW64_64Key"),
+])
+REGISTRY_ACCESS_MASK_STRINGS.update(COMMON_ACCESS_MASK_STRINGS)
+
+
+FILESYSTEM_ACCESS_MASK_MAPPING = [0x120089, 0x120116, 0x1200a0, 0x1f01ff]  # used in MapGenericMask
+FILESYSTEM_ACCESS_MASK_STRINGS = OrderedDict([
+    (0x1f01ff, "All Access"),
+    (0x1201bf, "Generic Read/Write/Execute"),
+    (0x12019f, "Generic Read/Write"),
+    (0x1200a9, "Generic Read/Execute"),
+    (0x1201b6, "Generic Write/Execute"),
+    (0x120089, "Generic Read"),
+    (0x120116, "Generic Write"),
+    (0x1200a0, "Generic Execute"),
+    (0x1, "Read Data/List Directory"),
+    (0x2, "Write Data/Add File"),
+    (0x4, "Append Data/Add Subdirectory/Create Pipe Instance"),
+    (0x8, "Read EA"),
+    (0x10, "Write EA"),
+    (0x20, "Execute/Traverse"),
+    (0x40, "Delete Child"),
+    (0x80, "Read Attributes"),
+    (0x100, "Write Attributes"),
+])
+FILESYSTEM_ACCESS_MASK_STRINGS.update(COMMON_ACCESS_MASK_STRINGS)
+
+
+def _get_access_mask_string(access_mask, mappings, access_strings):
+    """Return a string that describes the access mask.
+    :param access_mask: the access mask value
+    :param mappings: the mapping that is given to MapGenericMask
+    :param access_strings: the string for every mask option
+    """
+    if access_mask & 0x80000000:
+        access_mask |= mappings[0]
+    if access_mask & 0x40000000:
+        access_mask |= mappings[1]
+    if access_mask & 0x20000000:
+        access_mask |= mappings[2]
+    if access_mask & 0x10000000:
+        access_mask |= mappings[3]
+
+    string = _get_mask_string(access_mask, access_strings, ", ")
+    if string == '':
+        return "None 0x{:x}".format(access_mask)
+    return string
+
+
+def get_registry_access_mask_string(access_mask):
+    return _get_access_mask_string(access_mask, REGISTRY_ACCESS_MASK_MAPPING, REGISTRY_ACCESS_MASK_STRINGS)
+
+
+def get_filesystem_access_mask_string(access_mask):
+    return _get_access_mask_string(access_mask, FILESYSTEM_ACCESS_MASK_MAPPING, FILESYSTEM_ACCESS_MASK_STRINGS)
+
+
+FILESYSTEM_CREATE_OPTIONS = OrderedDict([
+    (0x1, "Directory"),
+    (0x2, "Write Through"),
+    (0x4, "Sequential Access"),
+    (0x8, "No Buffering"),
+    (0x10, "Synchronous IO Alert"),
+    (0x20, "Synchronous IO Non-Alert"),
+    (0x40, "Non-Directory File"),
+    (0x80, "Create Tree Connection"),
+    (0x100, "Complete If Oplocked"),
+    (0x200, "No EA Knowledge"),
+    (0x400, "Open for Recovery"),
+    (0x800, "Random Access"),
+    (0x1000, "Delete On Close"),
+    (0x2000, "Open By ID"),
+    (0x4000, "Open For Backup"),
+    (0x8000, "No Compression"),
+    (0x100000, "Reserve OpFilter"),
+    (0x200000, "Open Reparse Point"),
+    (0x400000, "Open No Recall"),
+    (0x800000, "Open For Free Space Query"),
+    (0x10000, "Open Requiring Oplock"),
+    (0x20000, "Disallow Exclusive"),
+])
+
+
+def get_filesysyem_create_options(options_mask):
+    return _get_mask_string(options_mask, FILESYSTEM_CREATE_OPTIONS, ", ")
+
+
+FilesystemCreateAttributes = OrderedDict([
+    (0x1, "R"),
+    (0x2, "H"),
+    (0x4, "S"),
+    (0x10, "D"),
+    (0x20, "A"),
+    (0x40, "D"),
+    (0x80, "N"),
+    (0x100, "T"),
+    (0x200, "SF"),
+    (0x400, "RP"),
+    (0x800, "C"),
+    (0x1000, "O"),
+    (0x2000, "NCI"),
+    (0x4000, "E"),
+    (0x10000, "V"),
+])
+
+
+def get_filesysyem_create_attributes(create_mask):
+    if 0 == create_mask:
+        return "n/a"
+    return _get_mask_string(create_mask, FilesystemCreateAttributes, "")
+
+
+FilesystemCreateShareMode = OrderedDict([
+    (0x1, "Read"),
+    (0x2, "Write"),
+    (0x4, "Delete"),
+])
+
+
+def get_filesysyem_create_share_mode(share_mask):
+    if 0 == share_mask:
+        return "None"
+    return _get_mask_string(share_mask, FilesystemCreateShareMode, ", ")
+
+
+FilesystemIoFlags = OrderedDict([
+    (0x10, "Buffered"),
+    (0x1, "Non-cached"),
+    (0x2, "Paging I/O"),
+    (0x4, "Synchronous"),
+    (0x40, "Synchronous Paging I/O"),
+    (0x400000, "Write Through"),
+])
+
+
+def get_filesysyem_io_flags(flags):
+    return _get_mask_string(flags, FilesystemIoFlags, ", ")
+
+
+FilesystemPriority = {
+    0: '',
+    1: 'Very Low',
+    2: 'Low',
+    3: 'Normal',
+    4: 'High',
+    5: 'Critical',
+}
+
+
+
+class RegistryTypes(enum.IntEnum):
+    REG_NONE = 0  # No value type
+    REG_SZ = 1  # Unicode nul terminated string
+    REG_EXPAND_SZ = 2  # Unicode nul terminated string
+    REG_BINARY = 3  # Free form binary
+    REG_DWORD = 4  # 32-bit number
+    REG_DWORD_LITTLE_ENDIAN = 4  # 32-bit number (same as REG_DWORD)
+    REG_DWORD_BIG_ENDIAN = 5  # 32-bit number
+    REG_LINK = 6  # Symbolic Link (unicode)
+    REG_MULTI_SZ = 7  # Multiple Unicode strings
+    REG_RESOURCE_LIST = 8  # Resource list in the resource map
+    REG_FULL_RESOURCE_DESCRIPTOR = 9  # Resource list in the hardware description
+    REG_RESOURCE_REQUIREMENTS_LIST = 10
+    REG_QWORD = 11  # 64-bit number
+    REG_QWORD_LITTLE_ENDIAN = 11  # 64-bit number (same as REG_QWORD)
+
+
+class RegistryKeyValueInformationClass(enum.IntEnum):
+    KeyValueBasicInformation = 0,
+    KeyValueFullInformation = 1,
+    KeyValuePartialInformation = 2
+    KeyValueFullInformationAlign64 = 3,
+    KeyValuePartialInformationAlign64 = 4,
+    KeyValueLayerInformation = 5,
+
+
+class RegistryKeyInformationClass(enum.IntEnum):
+    Basic = 0
+    Node = 1
+    Full = 2
+    Name = 3
+    Cached = 4
+    Flags = 5
+    Virtualization = 6
+    HandleTags = 7
+    Trust = 8
+    Layer = 9
+
+
+RegistryKeySetInformationClass = {
+    0: "KeyWriteTimeInformation",
+    1: "KeyWow64FlagsInformation",
+    5: "KeySetHandleTagsInformation",
+}
+
+
+class RegistryDisposition(enum.IntEnum):
+    REG_CREATED_NEW_KEY = 1
+    REG_OPENED_EXISTING_KEY = 2
+
+
+class FilesystemDisposition(enum.IntEnum):
+    Supersede = 0
+    Open = 1
+    Create = 2
+    OpenIf = 3
+    Overwrite = 4
+    OverwriteIf = 5
+
+
+class FilesystemOpenResult(enum.IntEnum):
+    Superseded = 0
+    Opened = 1
+    Created = 2
+    Overwritten = 3
+    Exists = 4
+    DoesNotExist = 5
